import Cocoa
import CPAPlugInManager

extension CPATestPlugInWindowController {
    @objc static let kTestPUsesColor: String = "com_tastycocoabytes_cpa_http_color_use"
    @objc static let kTestPColor: String = "com_tastycocoabytes_cpa_http_color"
}

public class CPATestPlugInWindowController: NSWindowController, CPAPluginPrincipalClassProtocol, CPAPluginPreferencesProtocol {
    @IBOutlet var useColoring: NSButton!
    @IBOutlet var colorWell: NSColorWell!
    @objc public func protocolColor() -> NSColor {
        if let data = UserDefaults.standard.object(forKey: CPATestPlugInWindowController.kTestPColor) as? Data {
            guard let protocolColor = try? NSKeyedUnarchiver.unarchivedObject(ofClass: NSColor.self, from: data) else { return .systemRed }
            return protocolColor
        }
        return .systemRed
    }

    public required convenience init() {
        self.init(plugInManager: CPAPlugInManagement.sharedManager)
    }

    public required init!(plugInManager: CPAPlugInManagement!) {
        super.init(window: nil)
        if UserDefaults.standard.object(forKey: CPATestPlugInWindowController.kTestPUsesColor) == nil {
            UserDefaults.standard.set(false, forKey: CPATestPlugInWindowController.kTestPUsesColor)
        }
        if UserDefaults.standard.object(forKey: CPATestPlugInWindowController.kTestPColor) == nil {
            guard let data = try? NSKeyedArchiver.archivedData(withRootObject: NSColor.systemBlue.withAlphaComponent(0.70), requiringSecureCoding: true) else { return }
            UserDefaults.standard.set(data, forKey: CPATestPlugInWindowController.kTestPColor)
        }

        UserDefaults.standard.addObserver(self, forKeyPath: CPATestPlugInWindowController.kTestPUsesColor, options: .new, context: nil)
        UserDefaults.standard.addObserver(self, forKeyPath: CPATestPlugInWindowController.kTestPColor, options: .new, context: nil)

        let pluginBundle = Bundle(for: CPATestPlugInWindowController.self)

        var options = [CPAPlugInManager.CPAPlugInColumn]()

        let aCol = CPAPlugInColumn(
            columnTitle: NSLocalizedString("HTTP Method", tableName: "LocalizableTestPlugIn", bundle: pluginBundle, value: "", comment: "HTTP Column Name"),
            keyPath: "httpMethodString",
            bindingOptions: nil,
            predicateOptions: [
                NSBindingOption.displayName: NSLocalizedString("HTTP Method contains", tableName: "LocalizableTestPlugIn", bundle: pluginBundle, value: "", comment: "HTTP Predicate Name"),
                NSBindingOption.predicateFormat: "httpMethodString CONTAINS[c] $value", CPAFindPredicateLeftExpression: NSExpression(forKeyPath: "httpMethodString"),
                CPAFindPredicateDisplayName: NSLocalizedString("HTTP Method", tableName: "LocalizableTestPlugIn", bundle: pluginBundle, value: "", comment: "HTTP Find Predicate Displayname"),
                CPAFindPredicateAttrType: String.self
            ]
        )
        options.append(aCol)

        let plugProtoHttp = CPAPlugInNetworkProtocol(protocolName: "HTTP", protocolPorts: [80, 443], sourceProtocol: .kCPAProtocolTCP, ethertype: nil, linktype: nil)

        let plugStrct = CPAPlugInStructure()
        plugStrct.plugInTitle = CPATestPlugInWindowController.self.pluginTitle() ?? ""
        plugStrct.plugInDescription = CPATestPlugInWindowController.self.pluginDescription() ?? ""
        plugStrct.plugInVersion = CPATestPlugInWindowController.self.pluginVersion()
        plugStrct.plugInAuthor = CPATestPlugInWindowController.self.pluginAuthor() ?? ""
        plugStrct.plugInPrincipalClass = CPATestPlugInWindowController.self as AnyClass
        plugStrct.plugInPreferencesInstance = self
        plugStrct.plugInAnalyzerClass = CPATestProtocolAnalyzer.self as AnyObject
        plugStrct.plugInTargetProtocols = [plugProtoHttp]
        plugStrct.plugInColumnKeyBindings = options

        _ = plugInManager.register(plugIn: plugStrct, forBundle: pluginBundle)

        loadWindow()
    }

    @available(*, unavailable)
    required init?(coder: NSCoder) {
        fatalError("coder has not been implemented")
    }

    deinit {
        UserDefaults.standard.removeObserver(self, forKeyPath: CPATestPlugInWindowController.kTestPUsesColor)
        UserDefaults.standard.removeObserver(self, forKeyPath: CPATestPlugInWindowController.kTestPColor)
    }

    override public var windowNibName: NSNib.Name? {
        "CPATestPlugInWindowController"
    }

    override public func loadWindow() {
        super.loadWindow()
        useColoring.bind(.value, to: UserDefaults.standard, withKeyPath: CPATestPlugInWindowController.kTestPUsesColor, options: nil)
        if #available(macOS 10.14, *) {
            colorWell.bind(.value, to: UserDefaults.standard, withKeyPath: CPATestPlugInWindowController.kTestPColor, options: [NSBindingOption.valueTransformerName: "CPAColorValueTransformer"])
        } else {
            colorWell.bind(.value, to: UserDefaults.standard, withKeyPath: CPATestPlugInWindowController.kTestPColor, options: [NSBindingOption.valueTransformerName: NSValueTransformerName.keyedUnarchiveFromDataTransformerName])
        }
    }

    public class func pluginVersion() -> String! {
        guard let buildV = Bundle(for: self).infoDictionary?[kCFBundleVersionKey as String] as? String,
              let versShort = Bundle(for: self).infoDictionary?["CFBundleShortVersionString"] as? String else { return "0" }
        return "\(versShort) (\(buildV))"
    }

    public class func pluginAuthor() -> String? {
        NSLocalizedString("Jens Francke", tableName: "LocalizableTestPlugIn", bundle: Bundle(for: CPATestPlugInWindowController.self), value: "", comment: "Plugin Author")
    }

    public class func pluginTitle() -> String? {
        NSLocalizedString("HTTP Analyzer", tableName: "LocalizableTestPlugIn", bundle: Bundle(for: CPATestPlugInWindowController.self), value: "", comment: "Plugin Title")
    }

    public class func pluginDescription() -> String! {
        NSLocalizedString("HTTP Protocol Test Plugin Analyzer", tableName: "LocalizableTestPlugIn", bundle: Bundle(for: CPATestPlugInWindowController.self), value: "", comment: "Plugin Description")
    }

    // MARK: -

    // MARK: Preferences

    public func usesColor() -> Bool {
        UserDefaults.standard.bool(forKey: CPATestPlugInWindowController.kTestPUsesColor)
    }

    public func preferencesSheet() -> NSWindow {
        window ?? NSWindow()
    }

    @IBAction func applyPrefsAction(_ sender: Any) {
        UserDefaults.standard.synchronize()
        NSApp.mainWindow?.endSheet(window!)
    }

    override public func windowDidLoad() {
        super.windowDidLoad()
        // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
    }

    // swiftlint: disable: block_based_kvo
    override public func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey: Any]?, context: UnsafeMutableRawPointer?) {
        if (keyPath == CPATestPlugInWindowController.kTestPUsesColor) || (keyPath == CPATestPlugInWindowController.kTestPColor) {
            NotificationCenter.default.post(name: NSNotification.Name(rawValue: kReloadPacketsView), object: self)
        } else {
            super.observeValue(forKeyPath: keyPath, of: object, change: change, context: context)
        }
    }
}
